import 'package:redstone/server.dart' as app;
import 'package:rethinkdb_driver/rethinkdb_driver.dart';
import 'package:rethinkdb_driver/rethinkdb_pool.dart';
import 'package:redstone_mapper/database.dart';
import 'package:connection_pool/connection_pool.dart';
import 'dart:async';

export 'package:connection_pool/connection_pool.dart';

app.RedstonePlugin rethinkPlugin(RethinkDbManager db, {String pattern: r'/.*'}) {

  return (app.Manager manager) {

    if (db == null)
      return;

    manager.addInterceptor(new app.Interceptor.conf(pattern), "database connection manager", (injector) async {

      RethinkConnection conn = await db.getConnection();
      app.request.attributes["dbConn"] = conn.conn;

      app.chain.next(() {
        db.closeConnection(conn, error: app.chain.error);
      });
    });
  };
}

class RethinkDbManager extends DatabaseManager<ManagedConnection<Connection>>{

  Rdb _pool;

  RethinkDbManager(String host, String database, {int port: 28015, String auth: "", int poolSize: 3}) {
    _pool = new Rdb(database, port, host, auth, poolSize);
  }

  @override
  void closeConnection(RethinkConnection connection, {error}) {
    var invalidConn = error is RqlDriverError;
    _pool.releaseConnection(
        connection,
        markAsInvalid: invalidConn);
  }

  @override
  Future<RethinkConnection> getConnection() async {
    ManagedConnection<Connection> mConn = await _pool.getConnection();
    return new RethinkConnection(mConn.connId, mConn.conn);
  }
}

class RethinkConnection extends ManagedConnection<Connection> implements Connection {

  RethinkConnection(int connId, Connection conn) : super (connId, conn);

  get isClosed {
    return conn.isClosed;
  }

  noreplyWait() {
    conn.noreplyWait();
  }


  void addListener(String key, Function val) {
    conn.addListener(key, val);
  }


  void on(String key, Function val) {
    conn.on(key, val);
  }


  close([bool noreply_wait]) {
    conn.close(noreply_wait);
  }


  Future<Connection> connect([bool noreply_wait]) {
    return conn.connect(noreply_wait);
  }


  use(String db) {

  }


}